package gssim.db.mysql;

import gssim.db.dao.data.Unit;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

public class UnitDAO extends AbstractDAO implements gssim.db.dao.UnitDAO {

	protected UnitDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(UnitDAO.class);
		setRelationName("unit");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM unit WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM unit WHERE (resource_id = ? OR ISNULL(?))"
				+ " AND (type = ? OR ISNULL(?))"
				+ " AND (name = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO unit(resource_id, type, name, amount,"
				+ "allocation_load, reservation_load) VALUES(?, ?, ?, ?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM unit WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE unit SET resource_id = ?, type = ?, name = ?, "
				+ "amount = ?, allocation_load = ?, reservation_load = ? "
				+ "WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(Unit obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(Unit obj, ResultSet resultSet)
			throws SQLException {

		ResultSetAdapter result = (ResultSetAdapter) resultSet;

		Integer pK = result.getIntObj("PK");
		Integer resourceId = result.getIntObj("resource_id");
		String type = result.getString("type");
		String name = result.getString("name");
		Integer amount = result.getIntObj("amount");
		Double allocationLoad = result.getDoubleObj("allocation_load");
		Double reservationLoad = result.getDoubleObj("reservation_load");

		obj.setPK(pK);
		obj.setResourceId(resourceId);
		obj.setType(type);
		obj.setName(name);
		obj.setAmount(amount);
		obj.setAllocationLoad(allocationLoad);
		obj.setReservationLoad(reservationLoad);

	}

	@Override
	public int fillStatementWithObject(PreparedStatement pstmt, Unit obj,
			int offset) throws SQLException {

		PreparedStatementAdapter stmt = (PreparedStatementAdapter) pstmt;

		stmt.setInt(offset + 1, obj.getResourceId());
		stmt.setString(offset + 2, obj.getType());
		stmt.setString(offset + 3, obj.getName());
		stmt.setInt(offset + 4, obj.getAmount());
		stmt.setDouble(offset + 5, obj.getAllocationLoad());
		stmt.setDouble(offset + 6, obj.getReservationLoad());

		return offset + 6;

	}

	@Override
	public List<Unit> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<Unit> list = new ArrayList<Unit>();

		stmt.setInt(1, finder.getInteger("resourceId"));
		stmt.setInt(2, finder.getInteger("resourceId"));
		stmt.setString(3, finder.getString("type"));
		stmt.setString(4, finder.getString("type"));
		stmt.setString(5, finder.getString("name"));
		stmt.setString(6, finder.getString("name"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			Unit obj = new Unit();
			fillObjectFromResultSet(obj, result);
			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(Unit obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		fillStatementWithObject(stmt, obj, 0);

		int result = stmt.executeUpdate();

		if (result == 1) {
			ResultSet gk = stmt.getGeneratedKeys();
			Integer pK = gk.next() ? gk.getInt(1) : -1;
			obj.setPK(pK);

			if (commit)
				commit();

			logSuccess("insert");

		} else
			logError("insert", result, obj);

		return result;
	}

	public int[] insertBatch(List<Unit> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			fillStatementWithObject(stmt, objList.get(i), 0);
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {

			ResultSet gk = stmt.getGeneratedKeys();
			for (int i = 0; i < objList.size(); i++) {
				Integer PK = gk.next() ? gk.getInt(1) : -1;
				objList.get(i).setPK(PK);
			}

			logSuccess("batch insert", j);

			if (commit)
				commit();

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public Unit select(Unit obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getPK());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();

		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			fillObjectFromResultSet(obj, result);
			return obj;
		}

		// no user found with specified id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(Unit obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();

		int offset = fillStatementWithObject(stmt, obj, 0);

		stmt.setInt(offset + 1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
